#!/bin/sh
# -*- sh -*-
# Installation script for ghs_elmd.

stdoutfile=${TMP:-/tmp}/stdout.$$

if [ "x$1" = "x-v" ]; then
    shift
    verbose=1
    exec 3>&1 4>&2 1>$stdoutfile
else
    exec 3>&1 4>&2 1>$stdoutfile 2>&1
fi

util_cleanup() {
    sed 's/^/STDOUT: /' $stdoutfile |dcat
    rm -f $stdoutfile
    if [ -n "$file_to_clean" ]; then
	rm -rf $files_to_clean
    fi
    exit
}

trap util_cleanup 0 2 15



######################################################################
######################################################################
### Predicates for installation program
######################################################################
######################################################################

exists() {
    ret=0
    for i in "$@"; do
	if [ ! -f "$i" -a ! -d "$i" ]; then
	    eecho -l "\"$i\" does not exist."
	    ret=1
	fi
    done
    return $ret
}

fileexists() {
    ret=0
    for i in "$@"; do
	if [ ! -f "$i" ]; then
	    eecho -l "\"$i\" does not exist or is not a regular file."
	    ret=1
	fi
    done
    return $ret
}

direxists() {
    ret=0
    for i in "$@"; do
	if [ ! -d "$i" ]; then
	    eecho -l "\"$i\" does not exist or is not a directory."
	    ret=1
	fi
    done
    return $ret
}

isexec() {
    ret=0
    for i in "$@"; do
	if [ ! -x "$i" ]; then
	    eecho -l "\"$i\" does not exist or is not an executable."
	    ret=1
	fi
    done
    return $ret
}



######################################################################
######################################################################
### Utility functions for a installation program
######################################################################
######################################################################

######################################################################
# Print out the arguments to the function.
args() {
    n=0
    uecho "args:"
    for i in "$@"; do
	n=`expr $n + 1`
	uecho "  $n:#$i#"
    done
}

######################################################################
# User echo
uecho() {
    echo "$*" >&3
}

######################################################################
# Error echo
# usage: eecho [-gl] stuff to echo to error output
# Normally, the error message is sent to the user.  If eecho logging
# is on, however, the mesage is redirected to a file.  The -g option
# overrides this behavior.  The -l option forces the message to either
# be logged to a file or be discarded if there is currently no
# logging.
eecho() {
    if [ -n "$verbose" ]; then
	echo "$*" >&2
    fi
    local=
    [ -n "$eecho_on_flag" -a -n "$eecho_file" ] && local=1
    if [ "$1" = "-g" ]; then
	local=
	shift
    elif [ "$1" = "-l" ]; then
	[ -z "$local" ] && return 0
	    # forced local messages are discarded when there is no
	    # local logging
	local=1
	shift
    fi

    if [ -n "$local" ]; then
	echo "$*" >> "$eecho_file"
    else
	echo "$*" >&4
    fi
}

######################################################################
# usage: eecho_log file
# Set the log file for eecho.  Also turns loggin of eecho on.
eecho_log() {
    eecho_clear
    if [ -n "$1" ]; then
	eecho_file="$1"
    else
	eecho_file=`tmpnam eecho`
    fi
    eecho_on_flag=1
}

######################################################################
# usage: eecho_dump
# Dump the logged error messages and turn off logging.
eecho_dump() {
    if [ -n "$eecho_file" -a -f "$eecho_file" ]; then
	ecat -p "$eecho_file"
	rm -f "$eecho_file"
    fi
    eecho_file=
    eecho_on_flag=
}

######################################################################
# usage: eecho_clear
# Discard all logged error messages and turn off logging.
eecho_clear() {
    if [ -n "$eecho_file" -a -f "$eecho_file" ]; then
	rm -f "$eecho_file"
    fi
    eecho_file=
    eecho_on_flag=
}

######################################################################
# usage: eecho_stop
# Stop logging error messages.
eecho_stop() {
    eecho_on_flag=
}

######################################################################
# Diagnostic echo
decho() {
    echo diag: "$*" >&4
}

######################################################################
# User cat
ucat() {
    cat "$@" >&3
}

######################################################################
# Error cat
# usage: ecat [-p] files_to_cat
# Prefix with ":> " if the -p option is given.
ecat() {
    if [ "$1" = "-p" ]; then
	shift
	cat "$@" | sed 's/^/:> /' >&4
    else
	cat "$@" >&4
    fi
}

######################################################################
# Diagnostic cat
dcat() {
    cat "$@" >&4
}

######################################################################
# Detect how to print a line with echo and not echo a terminating
# newline.  If neither method works, then neither will be used and
# newline will be printed.
setNecho() {
    NECHO="echo"
    nonl1=
    nonl2=

    test1="`printf %s good;printf %s bye`"
    test2="`$NECHO -n good;echo bye`"
    test3="`$NECHO good\\\\c;echo bye`"

    if [ "$test1" = "goodbye" ]; then
	#decho necho using printf
	NECHO=printf
	nonl1=%s
    elif [ "$test2" = "goodbye" ]; then
	#decho necho using -n
	nonl1=-n
    elif [ "$test3" = "goodbye" ]; then
	#decho necho using '\\c'
	nonl2='\c'
    #else
	#decho necho will use no arguments.
    fi
}

######################################################################
# Echo without a newline.  Must have setNecho() called first to
# configure it.
necho() {
    $NECHO $nonl1 "$*"${nonl2} >&3
}

######################################################################
# usage: system prog arg1 arg2...
# Run a program with correct stdin and stdout.
system() {
    "$@" 1>&3 2>&4
}

######################################################################
# usage: fatal_error msg
# Print a fatal error message and quit with non-zero exit status.
fatal_error() {
    eecho -g ""
    eecho -g "Fatal Error:  $*"
    eecho -g ""
    if [ -n "$eecho_file" ]; then
	eecho -g "Dump of last error messages:"
	eecho_dump
    fi
    exit 1
}

######################################################################
# Pause, prompt, and wait for a newline.
pause() {
    uecho ""
    if [ -n "$*" ]; then
	necho "$*"
    else
	necho "Press return to continue "
    fi
    read REPLY
    uecho ""
}

######################################################################
# Ask a question, present a default, and print the answer on stdout
# (note that the question is output on fd 3)
ask() {
    default="$2"
    if [ -n "$1" ]; then
	uecho "$1"
    fi
    necho "[$default]:"
    read REPLY
    if [ -z "$REPLY" ]; then
	echo "$default"
	return 0
    else
	echo "$REPLY"
	return 1
    fi
}

######################################################################
# usage: choose "prompt" default choice1 [choice2 [...]]
# Prompt the user to make a choice.  Check to make sure the choice is
# one of the acceptable choices.
choose() {
    pro="$1"
    default="$2"
    shift 2
    if [ -n "$pro" ]; then
	uecho "$pro"
    fi
    while /bin/true; do
	necho "[$default]:"
	read REPLY
	if [ -z "$REPLY" ]; then
	    echo "$default"
	    return 0
	fi
	for i in "$@"; do
	    if [ "$REPLY" = "$i" ]; then
		echo "$REPLY"
		return 0
	    fi
	done
	uecho ""
	uecho "Invalid choice."
	uecho "Valid choices are:"
	for i in "$@"; do
	    necho "\"$i\" "
	done
	uecho ""
    done
}

######################################################################
# usage: yn "prompt" default
# Ask a yes or no question with default.
yn() {
    case "$2" in
	[Yy]*)
	    pro="[Y/n]: "
	    default=0
	    ;;
	[Nn]*)
	    pro="[y/N]: "
	    default=1
	    ;;
	*)
	    pro="[y/n]: "
	    default=2
    esac
    while /bin/true; do
	if [ -n "$1" ]; then
	    uecho "$1"
	fi
	necho "$pro"
	read REPLY
	if [ -z "$REPLY" -a $default != 2 ]; then
	    return $default
	fi
	case "$REPLY" in
	    [Yy]*)
		return 0
		;;
	    [Nn]*)
		return 1
		;;
	    *)
		uecho "Invalid response.  Try again."
	esac
    done
}

######################################################################
# usage: basedir dir file1 file2...
# print the files as dir/file1 dir/file2...
basedir() {
    dir="$1"
    shift
    for i in "$@"; do
	echo "$dir/$i"
    done
}

######################################################################
# usage: checkfiles predicate file1 file2...
# Run predicate on each file until one fails (return 1) or all pass
# (return 0.)
checkfiles() {
    pred="$1"
    shift
    checkfilesret=0
    for i in "$@"; do
	$pred "$i" || checkfilesret=1
    done
    return $checkfilesret
}

######################################################################
# Check a directory.  Make sure that it exists and that it contains
# certain files.
# usage: dircheck directory file1 file2 ...
dircheck() {
    dir="$1"
    shift
    if [ ! -d "$dir" ]; then
	return 1
    fi
    checkfiles exists `basedir "$dir" "$@"`
    return $?
}

######################################################################
# checkerYN:  Run a command with a checking message.
# usage: checkerYN "msg" command arg1 arg2 ...
checkerYN() {
    msg="$1"
    shift
    necho "$msg..."
    eecho_log
    "$@"
    ret=$?
    eecho_stop
    if [ $ret = 0 ]; then
	uecho yes
    else
	uecho no
    fi
}

######################################################################
# checkerOK:  Run a command with a checking message.
# usage: checkerOK "msg" command arg1 arg2 ...
checkerOK() {
    msg="$1"
    shift
    necho "$msg..."
    eecho_log
    "$@"
    ret=$?
    eecho_stop
    if [ $ret = 0 ]; then
	uecho OK
    else
	uecho FAILED
    fi
    return $ret
}

######################################################################
# usage: createdir dir "failifexists"
# Create a directory and verify that it was created.  If
# "failifexists" is non-empty, then fail if it already exists.
createdir() {
    dir="$1"
    failifexists="$2"
    [ -d "$dir" -a -n "$failifexists" ] \
	&& { eecho -l "\"$dir\" already exists." ; return 1; }
    mkdir "$dir" \
	|| eecho -l "mkdir returned with an error on \"$dir\""
    [ -d "$dir" ] \
	|| { eecho -l "mkdir failed to create \"$dir\"";return 1;}
    return 0
}

######################################################################
# usage: safecp srcfile dstfile
# Copy srcfile to dstfile, being careful to check that you do not
# overwrite a file, and that the copy is successful.
safecp() {
    src="$1"
    dst="$2"
    [ ! -f "$src" ] \
	&& { eecho -l "safecp: source file \"$src\" does not exist"; return 1;}
    [ -f "$dst" ] \
	&& { eecho -l "safecp: dest file \"$dst\" already exists"; return 1;}
    cp "$src" "$dst" \
	&& eecho -l "safecp: cp returned with an error."
    [ ! -f "$dst" ] \
	&& { eecho -l "safecp: cp failed to create \"$dst\"";return 1;}
    return 0
}

######################################################################
# usage: safeln [-s] srcfile dstfile
# Link srcfile to dstfile, being careful to check that you do not
# overwrite a file, and that the link is successful.
safeln() {
    dashs=
    if [ "x$1" = "x-s" ]; then
	dashs="-s"
	shift
    fi
    src="$1"
    dst="$2"
    #[ ! -f "$src" ] \
	#&& { eecho -l "safeln: \"$src\" does not exist";return 1;}
    [ -f "$dst" ] \
	&& { eecho -l "safeln: \"$dst\" already exists";return 1;}
    ln $dashs "$src" "$dst" \
	&& eecho -l "safeln: ln returned an error"
    [ ! -f "$dst" ] \
	&& { eecho -l "safeln: failed to create \"$dst\"";return 1;}
    return 0
}

######################################################################
# usage: tmpnaminit
tmpnaminit() {
    for tnd in $TMP /tmp /var/tmp /temp; do
	#decho tnd:$tnd
	[ -d "$tnd" ] || continue;
	testfile="/tmp/util.sh.test.$$"
	[ ! -f "$testfile" ] || continue
	touch "$testfile"
	[ -f "$testfile" ] || continue
	rm -f "$testfile"
	[ ! -f "$testfile" ] || continue
	tmpnamdir="$tnd"
	return 0
    done
    eecho "tmpnaminit: Failed to find a suitable directory for temp files."
    return 1
}

######################################################################
# usage: tmpnam name del_on_exit
# Create a temporary file using "name" as a seed.
tmpnam() {
    if [ -z "$tmpnamdir" ]; then
	tmpnaminit
    fi
    if [ -z "$tmpnamdir" ]; then
	fatal_error "tmpnam failed to initialize"
    fi
    tmpnamcount=0
    while expr $tmpnamcount \< 100 >/dev/null 2>&1; do
	tmpfile="$tmpnamdir/tmpnam.$1.$tmpnamcount.$$"
	if [ ! -f "$tmpfile" ]; then
	    echo "$tmpfile"
	    if [ -n "$2" ]; then
		files_to_clean="$files_to_clean $tmpfile"
	    fi
	    return 0
	fi
	tmpnamcount=`expr $tmpnamcount + 1`
    done
    fatal_error "Couldn't create a tmp file."
}

strip_slash() {
    for i in "$@"; do
	echo "$i" | sed 's%\(.\)/$%\1%'
    done
}

make_absolute() {
    case "$1" in
	/*)
# [TRZ] Wed May  6 09:54:22 1998
# New Code
#
	    tmp=$1
# End New Code 
	    ;;
	*)
	    tmp=`pwd`/"$1"
    esac
    echo "$tmp"
}






######################################################################
######################################################################
## Initiailize
######################################################################
######################################################################

files_to_clean=

setNecho
tmpnaminit



intro_message() {
    ucat <<EOF

Note: Installing the license manager will take some knowledge of your
host and network environment.  You may need to consult a system
administrator for some of this information.  If this script is not run
as root, then you will not be able to install the init script so that
the license manager starts at boot.  In this case, you will have to
start it by hand whenever you restart your machine.

Green Hill Tools use the ghs_elmd license manager which came with your
tools distribution.  To use the tools, you will need a ghs_elmd running
somewhere on your local network that is serving licenses for the tools
you want to run.

When acquiring a license, the tools will broadcast onto the local
network looking for a license manager, and will check with any
responding ghs_elmd until they find one with an appropriate license.
Once the license manager is running, the granting and releasing of
licenses for the tools should be transparent to the user.  If you want
more information about the license manager, or it you want to change
it from the default configuration, then please read the documentation
sent to you by Green Hills Software, including the "Green Hills
License Manager: System Administrator's Guide" book.
EOF
}

existing_license_manager_message() {
    ucat <<EOF

If you just want to use one of the license servers discovered above,
then you may not need to install the license manager on this machine.
Note that if a ghs_elmd is already running on this machine, then you
should probably add the keys to the running license manager since you
can only run at most one ghs_elmd on a host.

EOF
}

summary_of_process_message() {
    ucat <<EOF

In order to run the license manager, you will need to do the following
(which will be explained in more detail later):
  1.  Create a keys directory.
  2.  Specify the key path.
  3.  Specify the log file.
  4.  Specify the maximum log file size.
  5.  Install the init script.
  6.  Start the license manager.
  7.  Generate license information.

EOF
}

key_directory_message() {
    ucat <<EOF

Key Directory
-------------

The keys directory is the directory where the keys for the license
server are installed.  If you have a keys directory or directories
from a previous installation of ghs_elmd and you would like to keep
using it, then you do not need to create one now.  The default keys
directory is a subdirectory "keys" in the installation directory where
the Green Hills tools reside.

EOF
}

key_path_message() {
    ucat <<EOF

Key Path
--------

ghs_elmd can read keys from several keys directories.  If you just
created a keys directory, then you will want to specify it (this is
the default.)  If you want to include several directories, then
separate the names with colons.  If you do not want to specify a keys
directory (so that ghs_elmd will use its default) then type none.

EOF
}

log_filename_message() {
    ucat <<EOF

Log File Name
-------------

ghs_elmd can create a log file and log all relevent information to
it.  This allows you to track problems should they occur, and can be
used to by the elmrpt command to show you usage statistics and other
information.  If you do not want a log file, enter "none" below.

EOF
}

log_file_size_message() {
   ucat <<EOF

Maximum Log File Size
---------------------

The log file size can be limited by specifying a maximum size.  When
this size is reached, the license manager will copy the log file from
$GHS_ELMD_LOG_FILE to $GHS_ELMD_LOG_FILE.old and truncate the
current log file.  The size can be an integer or floating point number
with no suffix (for bytes) or suffixed by "k" or "m" for kilobytes and
megabytes, respectively.

EOF
}

installing_init_script_message() {
   ucat <<EOF

Installing the Init Script
--------------------------

During boot up, UNIX systems have several ways of starting up various
services.  It is highly recommended that the license manager be added
to this boot up sequence so that it is automatically running when the
machine boots.  There are two common schemes for specifying services
to start on boot up.
1.  SYSV: with directories /etc/init.d and /etc/rc?.d
	In this method, there is a directory for each run level of the
	system.  A new service is added by placing a script in
	/etc/init.d and linking it into an /etc/rc directory
	(eg. /etc/rc2.d for multiuser.)  This method is common in
	System V derived systems.  Note:  Some systems use /sbin
	instead of /etc.  Choose SYSV for these systems as well.
2.  BSD: with files /etc/rc and /etc/rc.local
	In this method, there is one script for starting up all
	services.  This script is often called /etc/rc.local.  This
	method is common on BSD systems.
Important Note:  Care must be exercised when making changes to the
boot up sequence.  If you are not sure of what you are doing, then
talk to your system administrator.

EOF
}

bsd_init_script_message() {
   ucat <<EOF

If you want to add ghs_elmd to the init scripts, you or your system
administrator will have to do so by hand.  We recommend you look at
the script $SCRIPT for the commands to run.

EOF
}

sysv_init_script_message() {
   ucat <<EOF

Next you will be given a chance to edit the init script and then to
decide if you want to install it or not.

EOF
}

starting_license_manager_message() {
   ucat <<EOF

Starting the License Manager
----------------------------

The script $SCRIPT can be used to start and stop the
license manager.  To start it, type:
	$SCRIPT start
To stop it, type:
	$SCRIPT stop

EOF
}

#### Old part of general_license_info_message.  Removed until delivery
#### tests are added back.
##If the licenses you need are already installed (either on this machine
##or on a machine that will serve you licenses) then you can run the
##delivery tests now.

general_license_info_message() {
   ucat <<EOF

Generating License Information
------------------------------

This script can now generate a template file which you can edit and
then send to Green Hills Software through email or fax to request a
license.  It contains, among other things, the server code for this
machine.  You will need this code in order to request a key for a
license server running on this machine.  You can generate the server
code by hand for a host by running $GHS_INST/servecode on that host.

EOF
}





######################################################################
######################################################################
### Main functions for installation program
######################################################################
######################################################################

######################################################################
# Variables
elantoolsrequired="ghs_elmd Sghs_elmd.template elmadmin servecode"
instdirrequired="bugform.txt prodlist"
initd_etc_dir="/etc"


######################################################################
# usage: check_inst_dir
# Check if the $GHS_INST directory passes a few tests to see if it is
# the installation directory.
check_inst_dir() {
    if [ -n "$force" ]; then
	eecho -l "Accepting GHS_INST=$GHS_INST without testing."
	return 0
    fi
    checkerOK "Checking installation directory" \
	dircheck "$GHS_INST" $instdirrequired \
    || fatal_error "Installation directory missing files."
    return $?
}

######################################################################
# usage: do_check_lm_tools [basedir1 [basedir2 [...]]]
# Look in each base directory for the license manager tools.  Set
# GHS_ELM_DIR if they are found.
do_check_lm_tools() {
    if [ -n "$GHS_ELM_DIR" ]; then
	eecho -l "checking for elan tools in $GHS_ELM_DIR"
	if dircheck "$GHS_ELM_DIR" $elantoolsrequired; then
	    eecho -l "GHS_ELM_DIR=$GHS_ELM_DIR"
	    return 0
	else
	    fatal_error "'$GHS_ELM_DIR' doesn't contain the required lm tools."
	fi
    fi
    for bd in "$@"; do
	for testdir in `basedir $bd . elm elan lm`; do
	    eecho -l "checking for elan tools in $testdir"
	    if dircheck "$testdir" $elantoolsrequired; then
		GHS_ELM_DIR="$testdir"
		eecho -l "GHS_ELM_DIR=$GHS_ELM_DIR"
		return 0
	    fi
	done
    done
    return 1
}

######################################################################
# usage: check_lm_tools
# Look for the license manager tools.  Set GHS_ELM_DIR if they are found.
check_lm_tools() {
    if [ -n "$force" -a -n "$GHS_ELM_DIR" ]; then
	return 0;
    fi
    checkerOK "Checking for license manager tools" \
	do_check_lm_tools "$GHS_INST" \
    || fatal_error "Can't proceed without license manager tools."
    return $?
}

######################################################################
# usage: find_net_lm
# Look on the net for a license manager.
find_net_lm() {
    tmpfile=`tmpnam`
    necho "Checking the network for active license managers..."
    if $ELMADMIN -xs > "$tmpfile"; then
	uecho "OK"
    else
	uecho "FAILED"
    fi
    uecho ""
    ucat "$tmpfile"
    rm "$tmpfile"
}

######################################################################
# usage: do_generate_script
# generate a customized script from a template and from environment
# variables specifying the values to customize to.
do_generate_script() {
    src="$GHS_INST/Sghs_elmd.template"
    dst="$GHS_INST/Sghs_elmd"
    [ -f "$src" ] || { eecho "Init script template missing.";return 1;}
    tmpfile=`tmpnam script`
    cat > "$tmpfile" <<EOF
s%GHS_INST%$GHS_INST%g
s%GHS_ELM_DIR%$GHS_ELM_DIR%g
s%GHS_ELMD_KEY_DIR%$GHS_ELMD_KEY_DIR%g
s%GHS_ELMD_LOG_FILE%$GHS_ELMD_LOG_FILE%g
s%GHS_ELMD_LOG_SIZE%$GHS_ELMD_LOG_SIZE%g
EOF
    sed -f "$tmpfile" < "$src" > "$dst"
    [ -f "$dst" ] \
	|| { eecho "Failed to generate the init script.";return 1;}
    chmod 755 "$dst"
    [ -x "$dst" ] \
	|| { eecho "Failed to make the init script executable.";return 1;}
    rm -rf "$tmpfile"
    return 0
}

######################################################################
# usage: generate_script
# Customize the init script.
generate_script() {
    checkerOK "Generating (but not installing) the customized init script" \
	do_generate_script \
    || fatal_error "Failed to generate init script"
}

######################################################################
# usage: verify_sysv
# Check if this system uses sysv style init scripts.
verify_sysv() {
    for i in /etc /sbin; do
	[ -d "$i/init.d" ] || continue
	[ -d "$i/rc2.d" ] || continue
	initd_etc_dir="$i"
	return 0
    done
    # init scripts not found
    return 1
}

######################################################################
# usage: verify_bsd
# Check if this system uses bsd style init scripts.
verify_bsd() {
    [ -d /etc/rc.local ] || return 1
    return 0
}

######################################################################
# usage: do_detect_init_style
# Detect if the system has SYSV, BSD, or other style init scripts.
do_detect_init_style() {
    if verify_sysv; then
	if verify_bsd; then
	    init_style=unknown
	else
	    init_style=SYSV
	fi
    else
	if verify_bsd; then
	    init_style=BSD
	else
	    init_style=unknown
	fi
    fi
    return 0
}

######################################################################
# usage: detect_init_style
# Detect if the system has SYSV, BSD, or other style init scripts.
detect_init_style() {
    necho "Detecting system init script style..."
    do_detect_init_style
    uecho "$init_style"
}

######################################################################
# usage: do_install_init_script script initfile rcfile linktarget
# Carry out the installation of the init script.
do_install_init_script() {
    script="$1"
    initfile="$2"
    rcfile="$3"
    linktarget="$4"
    shift 4

    initfile_base=`basename "$initfile"`
    rcfile_base=`basename "$rcfile"`

    checkerYN "Checking if $initfile_base already exists" \
	dircheck "$initfile_base" \
	|| return 1
    #checkerYN "Checking if $initfile exists" \
	#checkfiles fileexists "$initfile" \
	#&& return 1
    checkerYN "Checking if $rcfile_base already exists" \
	dircheck "$rcfile_base" \
	|| return 1
    #checkerYN "Checking if $rcfile exists" \
	#checkfiles fileexists "$rcfile" \
	#&& return 1
    uecho ""
    checkerOK "Copying $script to $initfile" \
	safecp "$script" "$initfile" \
	|| return 1
    checkerOK "$symlink_word linking $linktarget to $rcfile" \
	safeln $symlink "$linktarget" "$rcfile" \
	|| return 1
    return 0
}

######################################################################
# usage: install_init_script script initfile rcfile linktarget
# Carry out the installation of the init script.
install_init_script() {
    script="$1"
    initfile="$2"
    rcfile="$3"
    linktarget="$4"
    shift 4

    [ -f "$initfile" ] && {
	eecho -g "\"$initfile\" already exists."
	abort=1
    }

    [ -f "$rcfile" ] && {
	eecho -g "\"$rcfile\" already exists."
	abort=1
    }

    if [ -n "$abort" ]; then
	eecho -g "You will have to move or remove those files before"
	eecho -g "you can install this init file."
	eecho -g "If you prefer, you can install the init file by hand."
	pause
	return 1
    fi

    do_install_init_script "$script" "$initfile" "$rcfile" "$linktarget" || {
	eecho -g "Error installing init script."
	eecho -g ""
	if [ -n "$eecho_file" ]; then
	    eecho -g "Dump of last error messages:"
	    eecho_dump
	fi
    }
}

######################################################################
# usage: generate_mail
# Generate an email request for a license.
generate_mail() {
    cat <<EOF > "$1"
This is a request for a license to use MULTI and the other Green Hills
tools.
Please submit it in one of the following ways:
    -  email to license@ghs.com
    -  fax to (805) 965-6343
    -  mail to Green Hills Software, 30 W. Sola St., Santa Barbara, CA,
93101
  
Name: Customer Name
Position: Customer's Position
Company: Customer's Company
Customer's Address
Customer's City, etc.
Email: email
Phone: Phone
Fax: Fax
  
I would like 1 Evaluation Floating license for:
EOF

    grep -i '^# *Title:' "$GHS_INST/prodlist" | \
	sed 's/^.*:/	/' >> "$1"

    cat <<EOF >> "$1"
to be installed on the machine named "`hostname`" running `uname -sr`
Server Code: `$GHS_INST/servecode`
EOF
}





######################################################################
######################################################################
### Start of main execution of install program.
######################################################################
######################################################################

prog=`basename $0`

usage() {
    ecat <<EOF
usage: $prog [-fh] inst_dir [lm_tools_dir]
Installs the license manager for Green Hills Tools.
    -h              Print this help message.
    -f              Forces the installation program to accept the
		    inst_dir argument even though it appears to be
		    missing some files.
    inst_dir        Pathname of the tools installation directory
		    (eg. /usr/green)
    lm_tools_dir    Pathname to directory containing license manager
		    tools (usually same as inst_dir.)
If lm_tools_dir is unspecified, then it will be automatically searched
for under inst_dir.

***Important note***: Here is how to use this installation program:
    1.  Make the directory where you want to install the tools:
	    mkdir /usr/green
    2.  Change to that directory: cd /usr/green
    3.  Untar the distribution to that directory:
	    tar xf /path/to/Green/Hills/distribution.tar
    4.  Run this program with the installation directory as an
	argument: sh $prog /usr/green
    From there, the program will prompt you for the information it
    needs to install the tools in the directory (/usr/green above.)
EOF
    exit 1
}

######################################################################
intro_message

pause

while /bin/true; do
    case "$1" in
	-f)
	    force=1
	    ;;
	-h)
	    usage
	    ;;
	*)
	    break;
    esac
    shift
done

if [ -z "$1" ]; then
    usage
else
    GHS_INST=`strip_slash "$1"`
    shift
fi

GHS_INST=`make_absolute "$GHS_INST"`

#decho "GHS_INST=$GHS_INST"

if [ -n "$1" ]; then
    GHS_ELM_DIR=`strip_slash "$1"`
    GHS_ELM_DIR=`make_absolute "$GHS_ELM_DIR"`
    shift
fi

check_inst_dir
check_lm_tools

ELMADMIN=$GHS_ELM_DIR/elmadmin

find_net_lm

######################################################################
existing_license_manager_message
yn "Do you want to skip the rest of the license manager installation?" y \
    && exit

######################################################################
summary_of_process_message

pause

make_script=1
SCRIPT="$GHS_INST/Sghs_elmd"
if [ -f "$SCRIPT" ]; then
    ######################################################################
    uecho "You already have a customized init script ($SCRIPT)"
    uecho "You can either keep it, or reconfigure and overwrite it."
    yn "Do you want to overwrite $SCRIPT?" n || make_script=
fi

if [ -n "$make_script" ]; then
    ######################################################################
    key_directory_message

    if yn "Do you want to create a keys directory" y; then
	GHS_ELMD_KEY_DIR=`ask "Enter the pathname for the keys directory?" \
	    "$GHS_INST/keys"`
	checkerOK "Creating the keys directory \"$GHS_ELMD_KEY_DIR\"" \
	    createdir "$GHS_ELMD_KEY_DIR" \
	|| fatal_error "Could not create keys directory."
    fi

    ######################################################################
    key_path_message

    GHS_ELMD_KEY_DIR=`ask "Enter the key path." "${GHS_ELMD_KEY_DIR:=none}"`

    if [ "$GHS_ELMD_KEY_DIR" = "none" ]; then
	GHS_ELMD_KEY_DIR=
    else
	GHS_ELMD_KEY_DIR="-e $GHS_ELMD_KEY_DIR"
    fi

    ######################################################################
    log_filename_message

    GHS_ELMD_LOG_FILE=`ask "Enter the log filename." "$GHS_INST/ghs_elmd.log"`

    if [ "$GHS_ELMD_LOG_FILE" = "none" ]; then
	GHS_ELMD_LOG_FILE=
    else
	GHS_ELMD_LOG_FILE="-l $GHS_ELMD_LOG_FILE"

	######################################################################
	log_file_size_message

	GHS_ELMD_LOG_SIZE=`ask \
	    "Enter the maximum log file size (or none for no limit)" \
	    "1.0m"`

	if [ "$GHS_ELMD_LOG_SIZE" = "none" ]; then
	    GHS_ELMD_LOG_SIZE=
	else
	    GHS_ELMD_LOG_SIZE="-m $GHS_ELMD_LOG_SIZE"
	fi
    fi
    generate_script
fi

######################################################################
installing_init_script_message

pause

detect_init_style

######################################################################
uecho ""
uecho "This system seems to have $init_style init scripts.  If this is"
uecho "incorrect, then please change it."
uecho ""

if [ "$init_style" = "unknown" ]; then
    init_style=other
fi

init_style=`choose "Please enter \"SYSV\", \"BSD\", or \"other\"" \
    "$init_style" "SYSV" "BSD" "other"`

if [ "$init_style" != "SYSV" ]; then
    ######################################################################
    bsd_init_script_message
else
    ######################################################################
    sysv_init_script_message

    yn "Do you want to edit $SCRIPT?" y \
    && system ${VISUAL:-${EDITOR:-vi}} "$SCRIPT"

    uecho "Do you want the init script $SCRIPT installed"
    uecho "into $initd_etc_dir/init.d and $initd_etc_dir/rc?.d for you?"
    uecho "(You will be prompted for file names spearately.)"
    if yn "" y; then
	uecho ""
	uecho "Next we need the name under which the init script should"
	uecho "be copied into $initd_etc_dir/init.d."
	initd_name=`ask "Enter the filename (without directory)" ghs_elmd`

	uecho ""
	uecho "The script must also be installed into $initd_etc_dir/rc?.d."
	rcdir=`ask \
	    "Which directory should it be installed in? (enter a number)" 2`

	uecho ""
	rc_name=`ask "Enter the script name for $initd_etc_dir/rc$rcdir.d" \
	    S99ghs_elmd`

	uecho ""
	uecho "We can either use a symbolic link (default) or a hard link"
	uecho "to link the script into $initd_etc_dir/rc$rcdir.d."
	symlink=-s
	yn "Do you want to use a symbolic link?" y || symlink=

	if [ -n "$symlink" ]; then
	    symlink_word=Symbolic
	else
	    symlink_word=Hard
	fi

	uecho "You have chosen the following:"
	uecho "    Copy $SCRIPT to $initd_etc_dir/init.d/$initd_name"
	uecho "    $symlink_word link $initd_etc_dir/rc$rcdir.d/$rc_name"
	uecho "        to ../init.d/$initd_name"

	if yn "Do you want to execute these choices?" y; then
	    install_init_script "$SCRIPT" "$initd_etc_dir/init.d/$initd_name" \
		"$initd_etc_dir/rc$rcdir.d/$rc_name" "../init.d/$initd_name"
	fi
    fi
fi

######################################################################
starting_license_manager_message

uecho "We can now start the license manager with the init script."
yn "Would you like to start the license manager?" y \
    && "$SCRIPT" start

uecho ""
######################################################################
general_license_info_message

if yn "Do you need to get keys from Green Hills?" y; then
    license_request=`\
	ask "Where should we save the template license request?" \
	    license_request.txt`
	generate_mail "$license_request"

    yn "Would you like to edit the license request now?" y \
	&& system ${VISUAL:-${EDITOR:-vi}} "$license_request"

    if yn "Would you like to email the license request now?" y; then
	mail license@ghs.com < "$license_request"
    else
	uecho "You can email it later to license@ghs.com of fax it"
	uecho "to (805)965-6343."
	uecho ""
	#uecho "When you have obtained and installed the licenses, you"
	#uecho "can run the delivery test by running the script:"
	#uecho "    $GHS_INST/delivery.sh"
	#uecho ""
    fi
fi

#yn "Would you like to run the delivery tests now?" y \
#    && "$GHS_INST/delivery.sh"

uecho ""
uecho `basename $0` "is now complete."
