/*
		    Low Level Interface Library
           Copyright 1983-1996 Green Hills Software,Inc.

 *  This program is the property of Green Hills Software, Inc,
 *  its contents are proprietary information and no part of it
 *  is to be disclosed to anyone except employees of Green Hills
 *  Software, Inc., or as agreed in writing signed by the President
 *  of Green Hills Software, Inc.
*/
/* ind_heap.c: Machine Independent low-level heap growth facilities. */

#include "indos.h"
#include <string.h>		/* for memset() */
#include "ind_thrd.h"

#if defined(EMBEDDED)
/******************************************************************************/
/* int brk (void *addr);						      */
/*  Extend the heap. Depending on the environment, this may just test for     */
/*  valid memory at addr-1 (like under the ROM Monitor) or it may result in   */
/*  process memory allocation (by a simulator process). Return 0 on success.  */
/*									      */
/*  Return -1 on an error and set errno = ENOMEM.			      */
/*									      */
/*  brk() is no longer called, and is only here for programs that want to use */
/*  it directly and know what they are doing. If possible, use only sbrk().   */
/******************************************************************************/
int brk (void *addr)
{
    if (__ghs_syscall(SYSCALL_BRK, addr)) {
	__gh_set_errno(ENOMEM);
	return -1;
    }
    return 0;
}

/******************************************************************************/
/* void *sbrk (int size);						      */
/*  Return a pointer to at least size bytes of contiguous read/write memory.  */
/*  The memory returned by sbrk on different calls need not be contiguous     */
/*  with each other (although they should be for compatibility with unix).    */
/*									      */
/*  Return -1 on an error and set errno = ENOMEM.			      */
/*									      */
/*  sbrk is normally only called by malloc.				      */
/*									      */
/*  WARNING: __ghs_alloc(), below, relies on our contiguous algorithm and our */
/*  behavior for size==0, which returns the next-available pointer.	      */
/******************************************************************************/
#if defined(__MC68000)
/*
 * this 68k version uses __ghssize_heap instead of __ghsend_heap because the 
 * latter may be out of the range of a 16-bit PC-relative data reference.
 * Instead, find the beginning of the heap, and add the size to determine the end.
 * This is tricky because __ghssize_heap is created by the linker, therefore the
 * size of the heap in bytes looks like a pointer in C!  If we simply take the
 * address of the symbol at runtime, it will be adjusted by the PC, which is wrong.
 * So, initialize a pointer to this value.  Also, disable picpidfixups which would
 * modify the value of 'theend' at startup.  Now, theend will contain the _size_
 * of the section at startup.  At our first call, add the beginning address to
 * determine the size correctly.  This code works in all addressing modes, but
 * requires the __ghssize_heap symbol which is only created by our 68k linker.
*/
#pragma ghs nopicpidfixups
void *sbrk (int size)
{
#pragma ghs startdata
    extern char __ghsbegin_heap[], __ghssize_heap[];
#pragma ghs enddata
    static char *pt;
    static char *theend = __ghssize_heap;	/* ignore the warning */
    char *npt;
    void *ret;

    __ghsLock();
    if (pt == NULL) {
	pt = __ghsbegin_heap;
	theend += ((int)pt);
    }
    npt = pt + size;
    if (npt <= theend)
	ret = ((pt = npt) - size);
    else {
	ret = (void *)-1;
	__gh_set_errno(ENOMEM);
    }
    __ghsUnlock();
    return ret;
}
#else
void *sbrk (int size)
{
#pragma ghs startdata
    extern char __ghsbegin_heap[], __ghsend_heap[];
#pragma ghs enddata
    static char *pt;
    char *npt;
    void *ret;

    __ghsLock();
    if (pt == NULL)
	pt = __ghsbegin_heap;
    npt = pt + size;
    if (npt <= __ghsend_heap)
	ret = ((pt = npt) - size);
    else {
	ret = (void *)-1;
	__gh_set_errno(ENOMEM);
    }
    __ghsUnlock();
    return ret;
}
#endif

/******************************************************************************/
/* void *__ghs_alloc (int size, int align);				      */
/*  Return a pointer to at least size bytes of contiguous read/write memory,  */
/*  at an address that is aligned to at least a multiple of align. This is a  */
/*  front end to sbrk(), mainly for internal use by the Low-Level Libraries.  */
/*									      */
/*  NOTE: 'align' MUST BE A POSITIVE POWER OF TWO. We also return differently.*/
/*									      */
/*  Return 0 on an error and set errno = ENOMEM.			      */
/******************************************************************************/
void *__ghs_alloc (int size, int align)
{
    char *m1 = (char *)-1;
    char *ret;

    __ghsLock();
    ret = sbrk(0);
    if (ret == m1)
	ret = 0;
    else {
	ret = sbrk((-(int)ret) & (align-1));	/* align our pointer */
	if (ret == m1 || sbrk(size) == m1)	/* do the real allocation */
	    ret = 0;
    }
    __ghsUnlock();
    return ret;
}

/******************************************************************************/
/* void *__ghs_calloc (int size, int align);				      */
/*  Allocate some memory by passing our arguments to __ghs_alloc(), then use  */
/*  memset() to clear the memory to 0 if the allocation was successful.	      */
/*									      */
/*  NOTE: 'align' MUST BE A POSITIVE POWER OF TWO.			      */
/******************************************************************************/
void *__ghs_calloc (int size, int align)
{
    void *p = __ghs_alloc(size, align);
    if (!p)
	return p;
    return memset(p, '\0', size);
}
#endif
